@@ -1,7 +1,7 @@
 //! Module that defines the Htmlify trait and implements it for items in this
 //! crate.
 
-use std::{borrow::Cow, fmt::Display};
+use std::{borrow::{Borrow, Cow}, fmt::Display};
 
 use crate::{Asset, AssetItem, Assets, Audio, Entity, Image, Mixin, Scene, Video};
 
@@ -12,12 +12,12 @@ use crate::{Asset, AssetItem, Assets, Audio, Entity, Image, Mixin, Scene, Video}
 /// ```ignore
 /// const TAG: &'static str;
 /// ```
-/// and the following optional defintions:
+/// and the following optional definitions:
 /// ```ignore
 /// fn attributes(&self) -> Vec<Attribute>;
 /// fn inner_html(&self) -> Cow<'static, str>;
 /// ```
-/// as well as the following definiton which should not need to be implemented, but may on occasion be useful to be overridden:
+/// as well as the following definition which should not need to be implemented, but may on occasion be useful to be overridden:
 /// ```ignore
 /// fn as_raw_html(&self) -> String 
 /// {
@@ -34,20 +34,37 @@ use crate::{Asset, AssetItem, Assets, Audio, Entity, Image, Mixin, Scene, Video}
 ///     )
 /// }
 /// ```
+/// Finally, the following may be called to get a more structured js_sys type:
+/// ```ignore
+/// fn as_element(&self) -> Option<web_sys::Element>;
+/// ```
 pub trait Htmlify
 {
-    const TAG: &'static str;
+    /// Sets the HTML tag associated when converting this to an element.
+    fn tag(&self) -> &'static str { "" }
+    /// Sets the attributes to include when converting this to an element.
     fn attributes(&self) -> Vec<Attribute>
     {
         vec!()
     }
+    /// Get the inner HTML
+    fn inner_html(&self) -> Vec<Box<dyn Htmlify>>
+    {
+        vec!()
+    }
+    /// Stringifies the inner HTML
+    fn inner_html_as_string(&self) -> String
+    {
+        self.inner_html().iter().map(|e| e.as_raw_html()).collect::<Vec<String>>().join("")
+    }
+    /// Convert this to a raw string of HTML
     fn as_raw_html(&self) -> String 
     {
         format!
         (
             "<{0} {2}> {1} </{0}>",
-            Self::TAG,
-            self.inner_html(),
+            self.tag(),
+            self.inner_html_as_string(),
             self.attributes()
                 .iter()
                 .map(Attribute::to_string)
@@ -55,9 +72,27 @@ pub trait Htmlify
                 .join(" ")
         )
     }
-    fn inner_html(&self) -> Cow<'static, str>
+    /// Convert this into a [web_sys::Element]
+    fn as_element(&self) -> Option<web_sys::Element>
     {
-        Cow::Borrowed("")
+        let document = web_sys::window().and_then(|win| win.document())?;
+        let element = document.create_element(self.tag()).ok()?;
+        for attribute in self.attributes()
+        {
+            element.set_attribute(attribute.name.borrow(), attribute.value.borrow()).ok()?;
+        }
+        for inner in self.inner_html()
+        {
+            if let "__STRING_MARKER" = inner.tag()
+            {
+                element.append_with_str_1(&inner.as_raw_html()).ok()?;
+            }
+            else
+            {
+                element.append_with_node_1(inner.as_element()?.as_ref()).ok()?;
+            }
+        }
+        Some(element)
     }
 }
 
@@ -93,7 +128,7 @@ impl Display for Attribute
 
 impl Htmlify for Scene
 {
-    const TAG: &'static str = "a-scene";
+    fn tag(&self) -> &'static str { "a-scene" }
     fn attributes(&self) -> Vec<Attribute>
     {
         self.components().iter()
@@ -101,21 +136,17 @@ impl Htmlify for Scene
             .chain(self.attributes().iter().map(Attribute::clone))
             .collect()
     }
-    fn inner_html(&self) -> Cow<'static, str>
+    fn inner_html(&self) -> Vec<Box<dyn Htmlify>>
     {
-        std::iter::once(self.assets().as_raw_html())
-            .chain(self.children()
-                .iter()
-                .map(|ent| ent.as_raw_html()))
-            .collect::<Vec<String>>()
-            .join("")
-            .into()
+        std::iter::once(Box::new(self.assets().clone()) as  Box<dyn Htmlify>)
+            .chain(self.children().iter().map(|child| Box::new(child.clone()) as Box<dyn Htmlify>))
+            .collect()
     }
 }
 
 impl Htmlify for Entity
 {
-    const TAG: &'static str = "a-entity";
+    fn tag(&self) -> &'static str { "a-entity" }
     fn attributes(&self) -> Vec<Attribute>
     {
         self.components().iter()
@@ -129,7 +160,7 @@ impl Htmlify for Entity
         (
             "<{0} {2}> {1} </{0}>",
             self.tag(),
-            self.inner_html(),
+            self.inner_html_as_string(),
             Htmlify::attributes(self)
                 .iter()
                 .map(Attribute::to_string)
@@ -137,15 +168,18 @@ impl Htmlify for Entity
                 .join(" ")
         )
     }
-    fn inner_html(&self) -> Cow<'static, str>
+    fn inner_html(&self) -> Vec<Box<dyn Htmlify>>
     {
-        self.children().iter().map(|ent| ent.as_raw_html()).collect::<Vec<String>>().join("").into()
+        self.children()
+            .iter()
+            .map(|child| Box::new(child.clone()) as Box<dyn Htmlify>)
+            .collect()
     }
 }
 
 impl Htmlify for Assets
 {
-    const TAG: &'static str = "a-assets";
+    fn tag(&self) -> &'static str { "a-assets" }
     fn attributes(&self) -> Vec<Attribute>
     {
         if self.timeout_ms <= 0
@@ -157,16 +191,25 @@ impl Htmlify for Assets
             vec!(Attribute::new("timeout", self.timeout_ms.to_string()))
         }
     }
-    fn inner_html(&self) -> Cow<'static, str>
+    fn inner_html(&self) -> Vec<Box<dyn Htmlify>>
     {
-        self.assets.iter().map(|a| a.as_raw_html()).collect::<Vec<String>>().join("").into()
+        self.assets.iter().map(|asset| Box::new(asset.clone()) as Box<dyn Htmlify>).collect()
     }
 }
 
 impl Htmlify for Asset
 {
-    /// This tag is unused
-    const TAG: &'static str = "a-asset";
+    fn tag(&self) -> &'static str 
+    { 
+        match self
+        {
+            Asset::Item(i) => i.tag(),
+            Asset::Image(i) => i.tag(),
+            Asset::Video(i) => i.tag(),
+            Asset::Audio(i) => i.tag(),
+            Asset::Mixin(i) => i.tag(),
+        }
+    }
     fn attributes(&self) -> Vec<Attribute>
     {
         self.into()
@@ -176,15 +219,8 @@ impl Htmlify for Asset
         format!
         (
             "<{0} {2}> {1} </{0}>",
-            match self
-            {
-                Self::Item(_) => AssetItem::TAG,
-                Self::Image(_) => Image::TAG,
-                Self::Video(_) => Video::TAG,
-                Self::Audio(_) => Audio::TAG,
-                Self::Mixin(_) => Mixin::TAG,
-            },
-            self.inner_html(),
+            self.tag(),
+            self.inner_html_as_string(),
             self.attributes()
                 .iter()
                 .map(Attribute::to_string)
@@ -196,7 +232,7 @@ impl Htmlify for Asset
 
 impl Htmlify for AssetItem
 {
-    const TAG: &'static str = "a-asset-item";
+    fn tag(&self) -> &'static str { "a-asset-item" }
     fn attributes(&self) -> Vec<Attribute>
     {
         vec!
@@ -209,7 +245,7 @@ impl Htmlify for AssetItem
 
 impl Htmlify for Image
 {
-    const TAG: &'static str = "img";
+    fn tag(&self) -> &'static str { "img" }
     fn attributes(&self) -> Vec<Attribute>
     {
         vec!
@@ -222,7 +258,7 @@ impl Htmlify for Image
 
 impl Htmlify for Video
 {
-    const TAG: &'static str = "video";
+    fn tag(&self) -> &'static str { "video" }
     fn attributes(&self) -> Vec<Attribute>
     {
         let mut attrs = vec!
@@ -241,7 +277,7 @@ impl Htmlify for Video
 
 impl Htmlify for Audio
 {
-    const TAG: &'static str = "audio";
+    fn tag(&self) -> &'static str { "audio" }
     fn attributes(&self) -> Vec<Attribute>
     {
         let mut attrs = vec!
@@ -260,7 +296,7 @@ impl Htmlify for Audio
 
 impl Htmlify for Mixin
 {
-    const TAG: &'static str = "a-mixin";
+    fn tag(&self) -> &'static str { "a-mixin" }
     fn attributes(&self) -> Vec<Attribute>
     {
         self.components.iter()
@@ -268,4 +304,12 @@ impl Htmlify for Mixin
             .chain(std::iter::once(Attribute::new("id", self.id.clone())))
             .collect()
     }
+}
+
+impl Htmlify for &str
+{
+    /// Used as a sentinal value
+    fn tag(&self) -> &'static str { "__STRING_MARKER" }
+    fn as_raw_html(&self) -> String { self.to_string() }
+    fn as_element(&self) -> Option<web_sys::Element> { None }
 }
\ No newline at end of file
